# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from rucio.client.pingclient import PingClient


def test_rucio_ping_rest(rest_client):
    """ RUCIO (REST): test a rucio ping """
    response = rest_client.get('/ping')
    assert response.status_code == 200
    ret = json.loads(response.get_data(as_text=True))
    assert 'version' in ret
    assert isinstance(ret, dict)


def test_rucio_ping():
    """ RUCIO (CLIENTS): test a rucio ping """
    client = PingClient()
    ret = client.ping()
    assert 'version' in ret
    assert isinstance(ret, dict)



# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import pytest

from rucio.common.utils import execute
from rucio.rse import rsemanager
from rucio.tests.common import load_test_conf_file, skip_rse_tests_with_accounts

from .rsemgr_api_test import MgrTestCases


@pytest.mark.noparallel(reason='creates and removes a test directory with a fixed name')
@skip_rse_tests_with_accounts
class TestRseGFAL2Impl(MgrTestCases):

    @classmethod
    @pytest.fixture(scope='class')
    def setup_rse_and_files(cls, vo, tmp_path_factory):
        """GFAL2 (RSE/PROTOCOLS): Creating necessary directories and files """

        cmd = "rucio list-rses --rses 'test_container_xrd=True'"
        exitcode, out, err = execute(cmd)
        rses = out.split()

        data = load_test_conf_file('rse_repository.json')
        prefix = data['WJ-XROOTD']['protocols']['supported']['xroot']['prefix']

        if len(rses) == 0:
            rse_name = 'WJ-XROOTD'
            hostname = data['WJ-XROOTD']['protocols']['supported']['xroot']['hostname']
        else:
            rse_name = 'XRD1'
            hostname = 'xrd1'
            prefix = '//rucio/'

        try:
            os.mkdir(prefix)
        except Exception as e:
            print(e)

        rse_settings, tmpdir, user = cls.setup_common_test_env(rse_name, vo, tmp_path_factory)
        rse_settings['protocols'][0]['impl'] = 'rucio.rse.protocols.gfal.Default'

        protocol = rsemanager.create_protocol(rse_settings, 'write')
        protocol.connect()

        os.system('dd if=/dev/urandom of=%s/data.raw bs=1024 count=1024' % prefix)

        for f in cls.files_remote:
            path = protocol.path2pfn(prefix + protocol._get_path('user.%s' % user, f))
            cmd = 'xrdcp %s/data.raw %s' % (prefix, path)
            execute(cmd)

        for f in MgrTestCases.files_local_and_remote:
            path = protocol.path2pfn(prefix + protocol._get_path('user.%s' % user, f))
            cmd = 'xrdcp %s/%s %s' % (tmpdir, f, path)
            execute(cmd)

        yield rse_settings, tmpdir, user

        clean_raw = '%s/data.raw' % prefix
        list_files_cmd_user = 'xrdfs %s ls %s/user.%s' % (hostname, prefix, user)
        clean_files = str(execute(list_files_cmd_user)[1]).split('\n')
        list_files_cmd_group = 'xrdfs %s ls %s/group.%s' % (hostname, prefix, user)
        clean_files += str(execute(list_files_cmd_group)[1]).split('\n')
        clean_files.append(clean_raw)
        for files in clean_files:
            clean_cmd = 'xrdfs %s rm %s' % (hostname, files)
            execute(clean_cmd)

        clean_prefix = '%s' % prefix
        list_directory = 'xrdfs %s ls %s' % (hostname, prefix)
        clean_directory = str(execute(list_directory)[1]).split('\n')
        clean_directory.append(clean_prefix)
        for directory in clean_directory:
            clean_cmd = 'xrdfs %s rmdir %s' % (hostname, directory)
            execute(clean_cmd)

    @pytest.fixture(autouse=True)
    def setup_obj(self, setup_rse_and_files, vo):
        rse_settings, tmpdir, user = setup_rse_and_files
        self.init(tmpdir=tmpdir, rse_settings=rse_settings, user=user, vo=vo)

    def test_delete_mgr_ok_dir(self):
        raise pytest.skip("Not implemented")



# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil

import pytest

from rucio.rse import rsemanager as mgr
from rucio.tests.common import load_test_conf_file, skip_rse_tests_with_accounts

from .rsemgr_api_test import MgrTestCases


def setup_posix_test_env(rse_name, rse_settings, user):
    """POSIX (RSE/PROTOCOLS): Creating necessary directories and files """

    data = load_test_conf_file('rse_repository.json')
    prefix = data[rse_name]['protocols']['supported']['file']['prefix']
    try:
        os.mkdir(prefix)
    except Exception as e:
        print(e)
    os.system('dd if=/dev/urandom of=%s/data.raw bs=1024 count=1024' % prefix)
    for f in MgrTestCases.files_remote:
        protocol = mgr.create_protocol(rse_settings, 'write')
        pfn = next(iter(mgr.lfns2pfns(rse_settings, {'name': f, 'scope': 'user.%s' % user}).values()))
        path = protocol.pfn2path(pfn)
        dirs = os.path.dirname(path)
        if not os.path.exists(dirs):
            os.makedirs(dirs)
        shutil.copy('%s/data.raw' % prefix, path)


@skip_rse_tests_with_accounts
class TestRsePOSIX(MgrTestCases):
    """
    Test the posix protocol
    """
    @classmethod
    @pytest.fixture(scope='class')
    def setup_rse_and_files(cls, tmp_path_factory, vo):
        """POSIX (RSE/PROTOCOLS): Removing created directorie s and files """
        rse_name = 'MOCK-POSIX'
        rse_settings, tmpdir, user = cls.setup_common_test_env(rse_name, vo, tmp_path_factory)
        setup_posix_test_env(rse_name, rse_settings, user)

        yield rse_settings, tmpdir, user

    @pytest.fixture(autouse=True)
    def setup_obj(self, setup_rse_and_files, vo):
        rse_settings, tmpdir, user = setup_rse_and_files
        self.init(tmpdir=tmpdir, rse_settings=rse_settings, user=user, vo=vo)


@skip_rse_tests_with_accounts
@pytest.mark.noparallel(reason='creates and removes a test directory with a fixed name')
class TestRsePOSIXSymlink(MgrTestCases):
    """
    Test the posix protocol using Symlink implementation
    """
    @classmethod
    @pytest.fixture(scope='class')
    def setup_rse_and_files(cls, tmp_path_factory, vo):
        """POSIX-SYMLINK (RSE/PROTOCOLS): Creating necessary directories and files """
        rse_name = 'MOCK-POSIX-SYMLINK'
        rse_settings, tmpdir, user = cls.setup_common_test_env(rse_name, vo, tmp_path_factory)
        setup_posix_test_env(rse_name, rse_settings, user)

        yield rse_settings, tmpdir, user

    @pytest.fixture(autouse=True)
    def setup_obj(self, setup_rse_and_files, vo):
        rse_settings, tmpdir, user = setup_rse_and_files
        self.init(tmpdir=tmpdir, rse_settings=rse_settings, user=user, vo=vo)



# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rucio.client.baseclient import BaseClient
from rucio.common.config import config_get, config_get_bool
from rucio.common.utils import generate_uuid
from rucio.tests.common import execute, get_long_vo


class TestReplicaHeaderRedirection:

    def test_replica_header_redirection(self, rse_factory, replica_client):
        rse1, rse1_id = rse_factory.make_rse(scheme='https', protocol_impl='rucio.rse.protocols.mock.Default')
        rse2, rse2_id = rse_factory.make_rse(scheme='https', protocol_impl='rucio.rse.protocols.mock.Default')

        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
            vo_header = '-H "X-Rucio-VO: %s"' % get_long_vo()
        else:
            vo_header = ''

        cacert = config_get('test', 'cacert')
        host = config_get('client', 'rucio_host')

        base_client = BaseClient()
        token = base_client.headers['X-Rucio-Auth-Token']

        """ REDIRECT: header to replica"""
        tmp_scope = 'mock'
        tmp_name = 'file_%s' % generate_uuid()
        cmd = 'curl -s -i --cacert %s -H "X-Rucio-Auth-Token: %s" %s -X GET %s/redirect/%s/%s''' % (cacert,
                                                                                                    token,
                                                                                                    vo_header,
                                                                                                    host,
                                                                                                    tmp_scope,
                                                                                                    tmp_name)
        _, out, _ = execute(cmd)
        assert '404 Not Found'.lower() in out.lower()

        replica_client.add_replicas(rse=rse1, files=[{'scope': tmp_scope,
                                                      'name': tmp_name,
                                                      'bytes': 1,
                                                      'adler32': '0cc737eb'}])
        replica_client.add_replicas(rse=rse2, files=[{'scope': tmp_scope,
                                                      'name': tmp_name,
                                                      'bytes': 1,
                                                      'adler32': '0cc737eb'}])
        _, out, _ = execute(cmd)
        assert '303 See Other'.lower() in out.lower()
        assert f'Location: https://{rse1_id}.cern.ch' in out \
               or f'Location: https://{rse2_id}.cern.ch' in out


class TestReplicaMetalinkRedirection:

    def test_replica_meta_redirection(self, rse_factory, replica_client):
        rse1, _ = rse_factory.make_rse(scheme='https', protocol_impl='rucio.rse.protocols.mock.Default')
        rse2, _ = rse_factory.make_rse(scheme='https', protocol_impl='rucio.rse.protocols.mock.Default')

        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
            vo_header = '-H "X-Rucio-VO: %s"' % get_long_vo()
        else:
            vo_header = ''

        cacert = config_get('test', 'cacert')
        host = config_get('client', 'rucio_host')

        base_client = BaseClient()
        token = base_client.headers['X-Rucio-Auth-Token']

        """ REDIRECT: metalink to replica"""
        tmp_scope = 'mock'
        tmp_name = 'file_%s' % generate_uuid()
        cmd = 'curl -s -i --cacert %s -H "X-Rucio-Auth-Token: %s" %s -X GET %s/redirect/%s/%s' % (cacert,
                                                                                                  token,
                                                                                                  vo_header,
                                                                                                  host,
                                                                                                  tmp_scope,
                                                                                                  tmp_name)
        _, out, _ = execute(cmd)
        assert '404 Not Found'.lower() in out.lower()

        replica_client.add_replicas(rse=rse1, files=[{'scope': tmp_scope,
                                                      'name': tmp_name,
                                                      'bytes': 1,
                                                      'adler32': '0cc737eb'}])
        replica_client.add_replicas(rse=rse2, files=[{'scope': tmp_scope,
                                                      'name': tmp_name,
                                                      'bytes': 1,
                                                      'adler32': '0cc737eb'}])
        _, out, _ = execute(cmd)
        assert '303 See Other'.lower() in out.lower()
        assert 'Link: </redirect/%s/%s/metalink' % (tmp_scope, tmp_name) in out

        cmd = 'curl -s -i --cacert %s -H "X-Rucio-Auth-Token: %s" %s -X GET %s/redirect/%s/%s/metalink' % (cacert,
                                                                                                           token,
                                                                                                           vo_header,
                                                                                                           host,
                                                                                                           tmp_scope,
                                                                                                           tmp_name)
        _, out, _ = execute(cmd)
        assert '200 OK'.lower() in out.lower()
        assert '<?xml' in out
        assert '<metalink' in out
        assert f'<url location="{rse1}"' in out
        assert f'<url location="{rse2}"' in out




'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '35.8.2',
    'branch_nick': 'release-35-LTS',
    'revision_id': 'b97fb8ca1f696c470b1ccb0ce21a09361f989e46',
    'revno': 13210
}



# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rucio.core import monitor


class TestMonitor:

    def test_record_counter_message(self, metrics_mock):
        """MONITOR (CORE): Send a counter message to graphite """
        monitor.MetricManager(prefix='test').counter('counter').inc(10)
        assert metrics_mock.get_sample_value('test_counter_total') == 10

    def test_record_gauge_message(self, metrics_mock):
        """MONITOR (CORE): Send a gauge message to graphite """
        monitor.MetricManager(prefix='test').gauge('gauge').set(10)
        assert metrics_mock.get_sample_value('test_gauge') == 10

    def test_record_timer_message(self, metrics_mock):
        """MONITOR (CORE): Send a timer message to graphite """
        monitor.MetricManager(prefix='test').timer('runtime').observe(500)
        assert metrics_mock.get_sample_value('test_runtime_count') == 1

    def test_context_record_timer(self, metrics_mock):
        """MONITOR (CORE): Send a timer message to graphite using context """

        metric_manager = monitor.MetricManager(prefix='test')
        with metric_manager.timer('context_timer'):
            var_a = 2 * 100
            var_a = var_a * 1
        assert metrics_mock.get_sample_value('test_context_timer_count') == 1

        with metric_manager.timer('context_timer'):
            var_a = 2 * 100
            var_a = var_a * 1
        assert metrics_mock.get_sample_value('test_context_timer_count') == 2
